#include <msp/gl/meshbuilder.h>
#include <msp/gl/texture2d.h>
#include "sidebysidecombiner.h"

namespace {

const char vs_source[] =
	"uniform float offset;\n"
	"varying vec2 texcoord;\n"
	"void main()\n"
	"{\n"
	"	gl_Position = vec4(gl_Vertex.x*0.5+offset, gl_Vertex.yzw);\n"
	"	texcoord = gl_Vertex.xy*0.5+0.5;\n"
	"}\n";

const char fs_source[] =
	"uniform sampler2D texture;\n"
	"varying vec2 texcoord;\n"
	"void main()\n"
	"{\n"
	"	gl_FragColor = texture2D(texture, texcoord);\n"
	"}\n";

}

namespace Msp {
namespace VR {

SideBySideCombiner::SideBySideCombiner(GL::View &v, bool c):
	view(v),
	mesh(GL::VERTEX2),
	shprog(vs_source, fs_source)
{
	target_width = view.get_width()/2;
	target_height = view.get_height()/2;
	render_aspect = static_cast<float>(target_width)/target_height;

	left_shdata.uniform("texture", 0);
	right_shdata.uniform("texture", 0);

	set_cross_eyed(c);

	GL::MeshBuilder bld(mesh);
	bld.begin(GL::TRIANGLE_STRIP);
	bld.vertex(-1, 1);
	bld.vertex(-1, -1);
	bld.vertex(1, 1);
	bld.vertex(1, -1);
	bld.end();
}

void SideBySideCombiner::set_cross_eyed(bool c)
{
	cross_eyed = c;
	float m = (cross_eyed ? -0.5f : 0.5f);
	left_shdata.uniform("offset", -m);
	right_shdata.uniform("offset", m);
}

void SideBySideCombiner::render(const GL::Texture2D &left, const GL::Texture2D &right) const
{
	GL::Bind bind_shprog(shprog);

	GL::Bind bind_tex(left);
	left_shdata.apply();
	mesh.draw();

	right.bind();
	right_shdata.apply();
	mesh.draw();

	view.get_context().swap_buffers();
}

} // namespace VR
} // namespace Msp
